"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var _ = require("underscore");
var formCore_1 = require("./formCore");
var formValueConverters_1 = require("./formValueConverters");
var EntityItems = (function () {
    function EntityItems() {
    }
    EntityItems.itemsId = function (items) {
        return function (value) { return _.find(items, function (i) { return i.id.toString() === value; }); };
    };
    EntityItems.itemsNumericId = function (items) {
        return function (value) { return _.find(items, function (i) { return i.id.toString() === value; }); };
    };
    return EntityItems;
}());
/**
The (abstract) FormBinder, derive from this calss to create your own custom FormBinder
TComponentProps: The type of the props required on the component being placed inside the form
TDataPropValue: The type of the data property being bound
TComponentPropValue: The type of the target property being bound on the component
*/
var FormBinderBase = (function () {
    function FormBinderBase(
        /** The name of the data property being bound  */
        dataName, 
        /** The name of the component property being bound  */
        propertySet, 
        /** The converter required to convert 'dataName' to 'propertySet' (TDataPropValue to TComponentPropValue) */
        valueConverter, 
        /** The name of the property being read from the change event currentTarget */
        propertyGet) {
        if (propertyGet === void 0) { propertyGet = propertySet; }
        this.dataName = dataName;
        this.propertySet = propertySet;
        this.valueConverter = valueConverter;
        this.propertyGet = propertyGet;
    }
    /** Sets the React elements properties required in binding */
    FormBinderBase.prototype.setElementProperty = function (props, dataBinder) {
        props["name"] = this.dataName;
        props[this.propertySet] = this.convert(dataBinder.getValue(this.dataName));
    };
    /** convert the value from element to data, set the dataBinder, and notify if changed */
    FormBinderBase.prototype.onChanged = function (dataBinder, newValue, notifyChanged) {
        var value = this.convertBack(newValue);
        if (_.isUndefined(value)) {
            return;
        }
        dataBinder.setValue(this.dataName, value);
        notifyChanged();
    };
    ;
    /** convert from data value to element property value */
    FormBinderBase.prototype.convert = function (data) {
        if (this.valueConverter && this.valueConverter.convert) {
            return this.valueConverter.convert(data);
        }
        return data;
    };
    /** convert from element property value to data value */
    FormBinderBase.prototype.convertBack = function (value) {
        if (this.valueConverter && this.valueConverter.convertBack) {
            return this.valueConverter.convertBack(value);
        }
        return value;
    };
    return FormBinderBase;
}());
exports.FormBinderBase = FormBinderBase;
/** A custom FormBinder that sets 'value' (of type T) and 'onValueChange: (value: T) => void' properties */
var ValueChangeBinder = (function (_super) {
    __extends(ValueChangeBinder, _super);
    function ValueChangeBinder() {
        _super.apply(this, arguments);
    }
    ValueChangeBinder.prototype.handleValueChanged = function (props, dataBinder, notifyChanged) {
        var _this = this;
        props.onValueChange = function (e) {
            _this.onChanged(dataBinder, e, notifyChanged);
        };
    };
    return ValueChangeBinder;
}(FormBinderBase));
exports.ValueChangeBinder = ValueChangeBinder;
/** An input FormBinder that sets native 'value' and 'onChange: (e) => void' properties */
var InputFormBinder = (function (_super) {
    __extends(InputFormBinder, _super);
    function InputFormBinder() {
        _super.apply(this, arguments);
    }
    InputFormBinder.prototype.handleValueChanged = function (props, dataBinder, notifyChanged) {
        var _this = this;
        props.onChange = function (e) {
            _this.onChanged(dataBinder, e.currentTarget[_this.propertyGet], notifyChanged);
        };
    };
    return InputFormBinder;
}(FormBinderBase));
exports.InputFormBinder = InputFormBinder;
/** A radio input FormBinder */
var RadioFormBinder = (function (_super) {
    __extends(RadioFormBinder, _super);
    function RadioFormBinder() {
        _super.apply(this, arguments);
    }
    RadioFormBinder.prototype.setElementProperty = function (props, dataBinder) {
        props["name"] = this.dataName;
        props[this.propertySet] = this.convert(dataBinder.getValue(this.dataName)) === props[this.propertyGet];
    };
    return RadioFormBinder;
}(InputFormBinder));
exports.RadioFormBinder = RadioFormBinder;
var KeyboardHelper = (function () {
    function KeyboardHelper() {
    }
    KeyboardHelper.getNumericRegEx = function (options) {
        if (options) {
            if (options.allowNegative) {
                if (options.decimals) {
                    return /[\d\.\-]/;
                }
                return /[\d\-]/;
            }
            if (options.decimals) {
                return /[\d\.]/;
            }
        }
        return /\d/;
    };
    KeyboardHelper.numericKeyPress = function (e, options) {
        var regEx = KeyboardHelper.getNumericRegEx(options);
        var m = e.key.match(regEx);
        if (!m) {
            e.preventDefault();
            return;
        }
        if (options) {
            var element = e.currentTarget;
            var selectionStart = element.selectionStart;
            var selectionEnd = element.selectionEnd;
            if (options.allowNegative) {
                var indexOfNegative = element.value.indexOf("-");
                var negativeWillRemove = indexOfNegative >= selectionStart && indexOfNegative < selectionEnd;
                if (m[0] === "-") {
                    if (negativeWillRemove) {
                        return;
                    }
                    if (indexOfNegative > -1 || selectionStart > 0) {
                        e.preventDefault();
                        return;
                    }
                }
                else if (!negativeWillRemove && selectionStart <= indexOfNegative) {
                    e.preventDefault();
                    return;
                }
            }
            if (options.decimals) {
                var indexOfDecimal = element.value.indexOf(".");
                var decimalWillRemove = indexOfDecimal >= selectionStart && indexOfDecimal < selectionEnd;
                if (m[0] === ".") {
                    if (decimalWillRemove) {
                        return;
                    }
                    if (indexOfDecimal > -1) {
                        e.preventDefault();
                        return;
                    }
                }
            }
        }
    };
    return KeyboardHelper;
}());
/** Form Binder helpers */
var FormBinder = (function () {
    function FormBinder() {
    }
    FormBinder.custom = function (formBinder) {
        return {
            "data-form-binder": formBinder
        };
    };
    /** hidden */
    FormBinder.hidden = function (dataName, valueConverter) {
        return this.defaultInputFormBinder(dataName, "hidden", valueConverter);
    };
    /** password */
    FormBinder.password = function (dataName, valueConverter) {
        return this.defaultInputFormBinder(dataName, "password", valueConverter);
    };
    /** text */
    FormBinder.text = function (dataName, valueConverter) {
        return this.defaultInputFormBinder(dataName, "text", valueConverter);
    };
    /** text 'email' */
    FormBinder.textEmail = function (dataName, valueConverter) {
        return this.defaultInputFormBinder(dataName, "email", valueConverter);
    };
    FormBinder.defaultInputFormBinder = function (dataName, type, valueConverter, propertySet) {
        if (propertySet === void 0) { propertySet = "value"; }
        var adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, propertySet, valueConverter));
        adaptorInjector["type"] = type;
        return adaptorInjector;
    };
    /** range */
    FormBinder.range = function (dataName, options) {
        var adaptorInjector = FormBinder.custom(new InputFormBinder(dataName, "value"));
        if (options) {
            adaptorInjector["min"] = options.min;
            adaptorInjector["max"] = options.max;
            adaptorInjector["step"] = options.step || 1;
        }
        return adaptorInjector;
    };
    /** uncontroller text input */
    FormBinder.defaultText = function (dataName, valueConverter) {
        return FormBinder.custom(new InputFormBinder(dataName, "defaultValue", valueConverter, "value"));
    };
    /** numeric text */
    FormBinder.textNumeric = function (dataName, options) {
        var converter = options ? new formValueConverters_1.NumericValueConverter(options) : formValueConverters_1.NumericValueConverter.instance;
        var adaptorInjector = this.defaultInputFormBinder(dataName, "text", converter);
        if (options) {
            adaptorInjector["min"] = options.min;
            adaptorInjector["max"] = options.max;
        }
        adaptorInjector[formCore_1.dataBinderAttribute].extender = function (props, ea, nc) {
            props.onKeyPress = function (e) { return KeyboardHelper.numericKeyPress(e, options); };
            props.onBlur = function (e) {
                e.currentTarget["value"] = converter.convert(ea.getValue(dataName));
            };
        };
        return adaptorInjector;
    };
    /** generic select */
    FormBinder.selectInject = function (dataName, valueConverter) {
        return FormBinder.custom(new InputFormBinder(dataName, "value", valueConverter));
    };
    /** select string */
    FormBinder.select = function (dataName) {
        return FormBinder.selectInject(dataName, formValueConverters_1.DefaultValueConverter.instance);
    };
    /** select number */
    FormBinder.selectNumeric = function (dataName) {
        return FormBinder.selectInject(dataName, formValueConverters_1.NumericValueConverter.instance);
    };
    FormBinder.selectId = function (dataName, items) {
        return FormBinder.selectInject(dataName, new formValueConverters_1.IdEntityValueConverter(_.isArray(items) ? EntityItems.itemsId(items) : items));
    };
    FormBinder.selectNumericId = function (dataName, items) {
        return FormBinder.selectInject(dataName, new formValueConverters_1.NumericIdEntityValueConverter(_.isArray(items) ? EntityItems.itemsNumericId(items) : items));
    };
    /** generic checkbox */
    FormBinder.checkboxInject = function (/** helooo */ dataName, valueConverter) {
        return this.defaultInputFormBinder(dataName, "checkbox", valueConverter, "checked");
    };
    /** checkbox binding to boolean */
    FormBinder.checkbox = function (dataName) {
        return FormBinder.checkboxInject(dataName);
    };
    /** checkbox binding to boolean derived from trueValue and falseValue equality testing */
    FormBinder.checkboxConvert = function (dataName, trueValue, falseValue) {
        return FormBinder.checkboxInject(dataName, new formValueConverters_1.CheckboxValueConverter(trueValue, falseValue));
    };
    /** generic input 'radio' */
    FormBinder.radioInject = function (dataName, value, valueConverter) {
        var adaptorInjector = FormBinder.custom(new RadioFormBinder(dataName, "checked", valueConverter, "value"));
        adaptorInjector["type"] = "radio";
        adaptorInjector["value"] = value;
        return adaptorInjector;
    };
    /** input 'radio' string value */
    FormBinder.radio = function (dataName, value) {
        return FormBinder.radioInject(dataName, value, formValueConverters_1.DefaultValueConverter.instance);
    };
    /** input 'radio' numeric value */
    FormBinder.radioNumeric = function (dataName, value) {
        return FormBinder.radioInject(dataName, value.toString(), formValueConverters_1.NumericValueConverter.instance);
    };
    /** input 'radio' item id string value */
    FormBinder.radioId = function (dataName, value) {
        return FormBinder.radioInject(dataName, value.id, new formValueConverters_1.IdValueConverter(function (v) { return value.id; }));
    };
    /** input 'radio' item id string value */
    FormBinder.radioIdEntity = function (dataName, value) {
        return FormBinder.radioInject(dataName, value.id, new formValueConverters_1.IdEntityValueConverter(function (v) { return value; }));
    };
    /** input 'radio' item id numeric value */
    FormBinder.radioNumericId = function (dataName, value) {
        return FormBinder.radioInject(dataName, value.id.toString(), new formValueConverters_1.NumericIdValueConverter(function (v) { return value.id; }));
    };
    /** input 'radio' item id numeric value */
    FormBinder.radioNumericIdEntity = function (dataName, value) {
        return FormBinder.radioInject(dataName, value.id.toString(), new formValueConverters_1.NumericIdEntityValueConverter(function (v) { return value; }));
    };
    return FormBinder;
}());
exports.FormBinder = FormBinder;
