"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require("react");
var classNames = require("classnames");
var formBinders_1 = require("../../form/formBinders");
var grid_1 = require("./../../layout/grid");
var form_1 = require("../form");
var dateHelpers_1 = require('./../../../utilities/dateHelpers');
var DateInput = (function (_super) {
    __extends(DateInput, _super);
    function DateInput() {
        _super.call(this);
        this.cId = "di_" + Math.random();
        this.state = { day: null, month: null, year: null, date: null };
    }
    DateInput.prototype.getDaysArrayByMonth = function () {
        return dateHelpers_1.DateHelpers.getDaysArrayByMonth(this.state.date, this.state.month);
    };
    DateInput.prototype.dayChanged = function (day) {
        var _this = this;
        this.setState({ day: parseInt(day) }, function () {
            _this.buildDate();
        });
    };
    DateInput.prototype.monthChanged = function (month) {
        var _this = this;
        this.setState({ month: parseInt(month) }, function () {
            _this.buildDate();
        });
    };
    DateInput.prototype.yearChanged = function (year) {
        var _this = this;
        this.setState({ year: parseInt(year) }, function () {
            _this.buildDate();
        });
    };
    DateInput.prototype.componentWillMount = function () {
        if (this.props.date) {
            this.setState(dateHelpers_1.DateHelpers.getDateParts(this.props.date));
        }
    };
    DateInput.prototype.componentWillReceiveProps = function (newProps) {
        if (newProps.date) {
            if (!this.state.date) {
                this.setState(dateHelpers_1.DateHelpers.getDateParts(newProps.date, true));
            }
            else {
                if (!dateHelpers_1.DateHelpers.areSame(this.state.date, newProps.date)) {
                    this.setState(dateHelpers_1.DateHelpers.getDateParts(newProps.date, true));
                }
            }
        }
        else {
            this.setState({ day: null, month: null, year: null, date: null });
        }
    };
    DateInput.prototype.buildDate = function () {
        var _this = this;
        var d = this.createDate();
        this.setState({ date: d }, function () {
            if (_this.props.onChange && d) {
                _this.props.onChange(_this.state.date);
            }
        });
    };
    DateInput.prototype.createDate = function () {
        return dateHelpers_1.DateHelpers.toDateFormat(this.state);
    };
    DateInput.prototype.render = function () {
        var _this = this;
        var dayOptions = [React.createElement("option", {value: "", disabled: true}, "Day")];
        this.getDaysArrayByMonth().forEach(function (d) {
            dayOptions.push(React.createElement("option", {key: _this.cId + "_day_" + d, value: d}, parseInt(d) < 10 ? "0" + d : d.toString()));
        });
        var monthOptions = [React.createElement("option", {value: "", disabled: true}, "Month")];
        monthOptions.push.apply(monthOptions, dateHelpers_1.DateHelpers.getMonthValues().map(function (v, i) { return React.createElement("option", {key: _this.cId + "_month_" + i, value: v.value}, v.label); }));
        var yearOptions = [React.createElement("option", {value: "", disabled: true}, "Year")];
        yearOptions.push.apply(yearOptions, dateHelpers_1.DateHelpers.getYearValues(this.props.futureDates, this.props.yearsFromNow).map(function (year) { return React.createElement("option", {key: _this.cId + "_year_" + year, value: year.toString()}, year); }));
        return React.createElement(form_1.Form, {className: classNames("date-input", this.props.disabled ? "input-disabled" : null), dataBinder: form_1.Form.jsonDataBinder(this.props.date), onDataChanged: function (d) { return _this.props.onChange(d); }}, React.createElement(grid_1.Grid, null, React.createElement(grid_1.Row, null, React.createElement(grid_1.Col, null, React.createElement("select", {disabled: this.props.disabled, onChange: function (e) { return _this.dayChanged(e.target.value); }, value: this.state.day ? this.state.day.toString() : ""}, dayOptions)), React.createElement(grid_1.Col, null, React.createElement("select", {disabled: this.props.disabled, onChange: function (e) { return _this.monthChanged(e.target.value); }, value: this.state.month ? this.state.month.toString() : ""}, monthOptions)), React.createElement(grid_1.Col, null, React.createElement("select", {disabled: this.props.disabled, onChange: function (e) { return _this.yearChanged(e.target.value); }, value: this.state.year ? this.state.year.toString() : ""}, yearOptions)))));
    };
    return DateInput;
}(React.Component));
exports.DateInput = DateInput;
var DateInputFormBinder = (function (_super) {
    __extends(DateInputFormBinder, _super);
    function DateInputFormBinder() {
        _super.apply(this, arguments);
    }
    DateInputFormBinder.customValue = function (dataName) {
        return new DateInputFormBinder(dataName, "date");
    };
    DateInputFormBinder.prototype.handleValueChanged = function (props, dataBinder, notifyChanged) {
        var _this = this;
        props.onChange = function (e) {
            _this.onChanged(dataBinder, e, notifyChanged);
        };
    };
    return DateInputFormBinder;
}(formBinders_1.FormBinderBase));
exports.DateInputFormBinder = DateInputFormBinder;
